"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const https = require("https");
const URL = require("url");
const codes_1 = require("./codes");
const defaultHeaders = {
    'Content-Type': 'application/json',
    'Accept': 'application/json'
};
function fetch(options) {
    return new Promise((resolve, reject) => {
        let resData = "";
        let url = URL.parse(options.url);
        let provider = url.protocol.charAt(url.protocol.length - 2) === 's' ? https : http;
        let data = options.data ? JSON.stringify(options.data) : '';
        let method = options.method ? options.method.toUpperCase() : 'GET';
        let headers = options.headers ? Object.assign(Object.assign({}, defaultHeaders), options.headers) : defaultHeaders;
        let wait = options.wait === false ? false : true;
        let timeout = wait === false ? 0 : options.timeout || 0;
        let timer;
        if (data)
            headers['Content-Length'] = Buffer.byteLength(data);
        let req = provider.request({
            hostname: url.hostname,
            port: url.port || 80,
            path: `${url.path}`,
            method: method,
            headers: headers
        }, (response) => {
            if (!wait)
                return resolve({ statusCode: codes_1.CODES.OK, data: true });
            let statusCode = response.statusCode;
            response.on('data', (chunk) => {
                resData += chunk;
            });
            response.on('end', () => {
                clearTimeout(timer);
                if (statusCode >= 200 && statusCode < 300) {
                    if (headers && headers.Accept === 'application/json')
                        resolve({ statusCode: statusCode, data: JSON.parse(resData) });
                    else
                        resolve({ statusCode: statusCode, data: resData });
                }
                else {
                    reject({ statusCode, error: JSON.parse(resData) });
                }
            });
        });
        req.on('error', (error) => {
            clearTimeout(timer);
            reject({ statusCode: codes_1.CODES.SERVIC_UNAVAILABLE, error: error.message || error });
        });
        req.write(data);
        req.end();
        if (timeout > 0)
            timer = setTimeout(() => {
                req.abort();
                reject({ statusCode: codes_1.CODES.GATEWAY_TIMEOUT, error: 'request taking too long' });
            }, timeout);
    });
}
exports.default = fetch;
//# sourceMappingURL=index.js.map