"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typesMap = {
    list: [Number, Boolean, String, Date, RegExp, Object, Array, Function],
    values: ['number', 'boolean', 'string', 'date', 'regexp', 'object', 'array', 'function']
};
const typesList = ['number', 'int', 'float', 'string', 'boolean', 'primitive', 'date', 'regexp', 'function', 'object', 'array', 'any'];
function searchMap(value) {
    if (typeof value === "function") {
        let index = typesMap.list.indexOf(value);
        if (index > -1)
            return typesMap.values[index];
    }
    else if (Array.isArray(value) && value.length === 1 && typeof value[0] === 'function') {
        let index = typesList.indexOf(value[0]);
        if (index > -1)
            return typesMap.values[index] + '[]';
    }
    return null;
}
function searchReverseMap(values) {
    let result = [];
    for (let value of values) {
        let index = typesMap.values.indexOf(value);
        if (index > -1)
            result.push(typesMap.list[index]);
    }
    return result;
}
exports.Types = {
    isValidType(type) {
        return typesList.indexOf(type) > -1;
    },
    number(value) {
        return typeof value === 'number';
    },
    int(value) {
        return (typeof value === 'number' && ("" + value).indexOf('.') === -1);
    },
    float(value) {
        return (typeof value === 'number' && ("" + value).indexOf('.') > -1);
    },
    string(value) {
        return typeof value === 'string';
    },
    boolean(value) {
        return typeof value === 'boolean';
    },
    primitive(value) {
        return (typeof value === 'number' || typeof value === 'string' || typeof value === 'boolean');
    },
    date(value) {
        return value instanceof Date;
    },
    regexp(value) {
        return value instanceof RegExp;
    },
    function(value) {
        return typeof value === 'function';
    },
    object(value) {
        return value && typeof value === "object" && value.toString() === "[object Object]";
    },
    array(value) {
        return Array.isArray(value);
    },
    any() { return true; },
    arrayOf(type, value) {
        let lng = type.length;
        if (type.charAt(lng - 1) === 's')
            type = type.slice(0, lng - 1);
        if (this.isValidType(type) && Array.isArray(value))
            return value.every((entry) => this[type](entry));
        return false;
    },
    getTypesOf(value) {
        let types = [];
        if (!Array.isArray(value)) {
            for (let i = 0; i < typesList.length; i++)
                if (this[typesList[i]](value))
                    types.push(typesList[i]);
        }
        else {
            types.push('array', 'any[]');
            for (let i = 0; i < typesList.length; i++)
                if (this.arrayOf(typesList[i], value))
                    types.push(typesList[i] + '[]');
            if (this.object(value[0])) {
                types.push(value[0].constructor.name + '[]');
                types.push('object[]');
            }
        }
        types.push(...searchReverseMap(types));
        if (types.indexOf("object") > -1) {
            types.push(value.constructor);
            types.push(value.constructor.name);
        }
        return types;
    }
};
//# sourceMappingURL=types.js.map