"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const parse_1 = require("./object/parse");
const stringify_1 = require("./string/stringify");
const str_splice_1 = require("./string/str-splice");
function cleanParam(param) {
    param = param.slice(1);
    if (param.charAt(param.length - 1) === '?')
        param = param.slice(0, param.length - 1);
    return param;
}
function cleanUrl(path) {
    return path
        ? path.trim()
            .replace(/^\/|\/$/g, "")
            .replace('://', '%%%')
            .replace(/\/$/g, "")
            .replace(/\/{2,}/g, "/")
            .replace(/(\w+)\/\.+/g, "$1")
            .replace('%%%', '://')
        : "";
}
class PathPattern {
    constructor(pattern) {
        this.pattern = pattern;
        this._params = null;
        this.blocks = this.pattern.split('/');
        this.paramsNames = this.blocks.length > 1 ? this.blocks.filter(block => block.charAt(0) === ':' || block === '*').map(param => cleanParam(param)) : [];
        this.hasRest = this.pattern.charAt(this.pattern.length - 1) === '*';
        this.hasOptional = this.pattern.charAt(this.pattern.length - 1) === '?';
        this.countOptionals();
    }
    countOptionals() {
        this.optionalsCount = 0;
        for (let block of this.blocks)
            if (this.hasRest && this.optionalsCount > 0)
                throw `invalid path pattern: ${this.pattern}`;
            else if (block.charAt(block.length - 1) === '?')
                this.optionalsCount++;
            else if (this.optionalsCount > 0)
                throw 'invalid path pattern';
    }
    get params() {
        return !!this._params ? Object.assign({}, this._params) : {};
    }
    match(path) {
        path = cleanUrl(path);
        let pathBlocks = path.split('/');
        let values = {};
        if (this.blocks.length > pathBlocks.length) {
            if (!this.hasOptional)
                return !!(this._params = null);
            if (this.blocks.length - pathBlocks.length > this.optionalsCount)
                return !!(this._params = null);
        }
        else if (this.blocks.length < pathBlocks.length && !this.hasRest)
            return !!(this._params = null);
        else {
            for (let [i, block] of this.blocks.entries())
                if (block.charAt(0) !== ':' && block !== '*')
                    if (block !== pathBlocks[i])
                        return !!(this._params = null);
        }
        for (let [i, block] of this.blocks.entries()) {
            if (block.charAt(0) === ':')
                values[cleanParam(block)] = pathBlocks[i] || null;
            else if (block === '*')
                values['*'] = pathBlocks.slice(i).join('/');
        }
        this._params = values;
        return true;
    }
    fill(values) {
        let path = this.pattern;
        for (let key in values) {
            if (key === '*')
                path = path.replace(/\*$/, values[key]);
            else
                path = path.replace(new RegExp(`:${key}\\??`), values[key]);
        }
        while (path.charAt(path.length - 1) === '?')
            path = path.slice(0, path.lastIndexOf('/'));
        return path;
    }
}
exports.PathPattern = PathPattern;
class URL extends url_1.URL {
    constructor(href, base) {
        super(href, base);
        this.query = this.search ? URL.QueryToObject(this.search.slice(1)) : {};
    }
    static QueryToObject(query) {
        query = decodeURI(query);
        let result = {};
        let parts = query.split('&');
        for (let i = 0; i < parts.length; i++) {
            let part = parts[i];
            let pair = part.split('=');
            result[pair[0]] = pair[1] ? parse_1.parse(pair[1]) : null;
        }
        return result;
    }
    static ObjectToQuery(obj, encode) {
        let result = "";
        if (!obj || !Object.keys(obj).length)
            return result;
        for (let prop in obj)
            result += '&' + prop + '=' + stringify_1.stringify(obj[prop]);
        return encode ? encodeURI(result.slice(1)) : result.slice(1);
    }
    static Clean(path) {
        return cleanUrl(path);
    }
    get secure() {
        return this.protocol.charAt(this.protocol.length - 2) === 's';
    }
    set secure(value) {
        if (value && !this.secure)
            this.protocol = str_splice_1.strSplice(this.protocol, this.protocol.length - 1, 0, 's');
        else if (!value && this.secure)
            this.protocol = str_splice_1.strSplice(this.protocol, this.protocol.length - 2, 1);
    }
}
exports.URL = URL;
//# sourceMappingURL=url.js.map